'*********************************************************************
'                            UPN_CALC.BAS
'                         Flash of Inspiration I
'
'(c) modified by Ing.Bro St.Lehmann, D-77756 Hausach, SL@iL-online.de
'*********************************************************************
rem  Demoprogramm fr LC-Anzeige, 12er Tastatur, Summer und Arithmetik  
rem      
rem      Einrichten der Platine
rem
rem      Tasten PortB -> durchschalten
rem      LCD an PortB -> glatt durchschalten
rem      Stift PortA,0 mit Stift LS verbinden (Summer an PortA,0)
rem
rem      Programm in PIC 16F628 laden und starten
rem
rem      Dieser UPN-Rechner kann nur in einem Zahlenbereich von 16 Bit
rem      arbeiten. Dementsprechend liegt ein Zahlenraum von 0 bis 65535
rem      zur Verfgung. In der ersten Zeile wird neben dem Top of Stack 
rem      auch der nchste Stackeintrag angezeigt. So sieht man die beiden
rem      Argumente die durch die Rechenoperation verknpft werden. Die 
rem      Tasten haben folgende Funktion: 
rem      # = ENTER; * = umschalten auf Funktionsebene
rem      in der Funktionsebene bedeutet die Taste 1 = +; Taste 2 = -;
rem      Taste 4 = *; Taste 5 = /; 
rem      Taste 8 = DUP; Taste 9 = SWAP X1 <-> X2; Taste # = POP;
rem
rem      --------------------------------------------------------------
rem      demonstrates how to work with LCD, keypad and buzzer
rem
rem      the following settings and connections must be done
rem
rem      put switches "Tasten PortB" all to on 
rem      put switches "LCD an PORTB" 1 .. 6 to on
rem      connect LS to PortA,0 (buzzer onto port RA,0)
rem      switches of S5 all off
rem
rem      load compiled program into "Flash Of Inspiration I"
rem
rem      when pressing a key its value is displayed on lcd and 
rem      a short sound is generated.


         $NCALDEF
  
'Watchdog off, all comparators off, MCLR- and OSC2-Pin are used for io
         define device 16f628
         config wdt_off
         config ircp_osc
         config cmcfg7
         config MCLR_INT
     
rem Variablen
'we use only two byte variables on ram location 20h and 21h
         define key1=$20 as byte
         define key2=$21 as byte
         define state=$22 as byte
         
         define x1=$26 as word
         define x2=$28 as word
         define x3=$2a as word
         define x4=$2c as word
         define xx=$2e as word
             
'conversation table for keypad. No keypressed returns "A", key 0 returns 11 etc.
'this table depends on how the keypad is connected to port 
         define keys rb,"A",11,7,4,1,0,8,5,2,12,9,6,3
       
         xtal 4.00                     'xtal frequency

         tris ra,0                     'all pins are output now
         tris rb,0                     'all pins are output now
         output ra,0                   'all pins have low level
         output rb,0                   'all pins have low level
         lcdinit rb,2,16,              'init. lcd with each 16 chars on 2 lines 

         lcdwrite 1,1,"UPN Calculator "          'when starting, show this text 
         lcdwrite 2,1,"(c) iL    V1.0a"
         wait 2000                               'for 2 second
         lcdwrite 1,1,"               "          'then delete the first line
         lcdwrite 2,1,"*->1+ 2- 4* 5/ "
         goto start                              'skip all subroutines

pushx:         
         let x4=x3                  
         let x3=x2
         let x2=x1
         let x1=0
         return
 
start:
         let x1=0                                'clear all registers
         let x2=0         
         let x3=0
         let x4=0
loop0:         
         let state=0                   'state machine
nokey:
         inkey key1
         if key1<>65 then goto nokey   'wait until no key is pressed
loop:         
         lcdwrite 1,1,x1,              'show first and second entry
         lcdwrite 1,11,x2
loopw:
         inkey key1                    'read keypad
         if key1=65 then goto loopw    'if no key is pressed then wait
         lofreq ra,0,800,100
         if state=2 then goto calc
         if key1=12 then goto push     'shove stack x1->x2->x3->x4
         if key1=11 then goto functions 'special funktion keys (+,-,*,/, etc.)
         if state=1 then goto loop1
         if x1<>0 then gosub pushx     'start of a new entry sequence
loop1:         
         let state=1                   '=entry mode
         let x1=x1*10
         let x1=x1+key1
         goto nokey

push:
         gosub pushx
         goto loop0
                  
pop0:
         let x1=x2
pop:
         let x2=x3
         let x3=x4
         let x4=0
         goto loop0
functions:
         let state=2                   'switch to calc mode
         goto nokey

calc:
         if key1=1 then goto add
         if key1=2 then goto sub
         if key1=4 then goto mult
         if key1=5 then goto div
         if key1=8 then goto dup       'double top of stack
         if key1=9 then goto swp       'swap x1 and x2
         if key1=12 then goto pop0
         goto nokey                    'no valid key is pressed
add:
         let x1=x2+x1
         goto pop                      'reorganize stack                                    
sub:
         let x1=x2-x1
         goto pop                      'reorganize stack                                    
mult:
         let x1=x2*x1
         goto pop                      'reorganize stack                                    
div:
         let x1=x2 / x1
         goto pop                      'reorganize stack                                    
dup:
         gosub pushx
         let x1=x2
         goto loop0
swp:
         let xx=x1
         let x1=x2
         let x2=xx
         goto loop0
         
         